@@ -12,7 +12,7 @@ from osgeo import gdal, ogr, osr
 import wradlib
 from wradlib import georef, util
 
-from . import has_data, requires_data
+from . import has_data, requires_data, requires_secrets
 
 np.set_printoptions(
     edgeitems=3,
@@ -244,8 +244,8 @@ class TestCoordinateTransform:
         filename = util.get_wradlib_data_file("misc/polar_dBZ_tur.gz")
         data = np.loadtxt(filename)
         # we need to have meter here for the georef function inside mip
-        d1 = np.arange(data.shape[1], dtype=np.float) * 1000
-        d2 = np.arange(data.shape[0], dtype=np.float)
+        d1 = np.arange(data.shape[1], dtype=np.float_) * 1000
+        d2 = np.arange(data.shape[0], dtype=np.float_)
         data = np.roll(data, (d2 >= angle).nonzero()[0][0], axis=0)
 
         # calculate max intensity proj
@@ -830,13 +830,15 @@ class TestGdal:
         data, coords, proj = georef.extract_raster_dataset(ds, mode="edge")
         assert coords.shape[-1] == 2
 
+    @requires_data
+    @requires_secrets
     def test_get_raster_elevation(self):
         # crop file using translate to keep download sizes minimal
         gdal.Translate(
             "/vsimem/clip.tif", self.geofile1, projWin=[5.5, 49.5, 5.6, 49.4]
         )
         ds = wradlib.io.open_raster("/vsimem/clip.tif")
-        georef.get_raster_elevation(ds, download={"region": "Eurasia"})
+        georef.get_raster_elevation(ds)
 
     def test_get_raster_extent(self):
 
@@ -854,11 +856,10 @@ class TestGdal:
         extent = georef.get_raster_extent(self.ds2, geo=True, window=False)
         np.testing.assert_array_almost_equal(extent, self.corner_geo_gdalinfo)
 
+    @requires_data
+    @requires_secrets
     def test_merge_raster_datasets(self):
-        download = {"region": "Eurasia"}
-        datasets = wradlib.io.dem.get_srtm(
-            [5, 6, 49.4, 49.5], merge=False, download=download
-        )
+        datasets = wradlib.io.dem.get_srtm([5, 6, 49.4, 49.5], merge=False)
         georef.merge_raster_datasets(datasets)
 
     def test_raster_to_polyvert(self):
